package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroup;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroupLeader;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkerGroupLeaderRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkerGroupRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Date: 2021/9/22.
 */
@Service
public class MarkerGroupLeaderService {

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @Autowired
    private MarkerGroupRepo markerGroupRepo;

    @Autowired
    private MarkerGroupLeaderRepo markerGroupLeaderRepo;

    /**
     * 创建分组
     *
     * @param markerGroup
     * @param stage
     * @return
     */
    @Transactional
    public MarkerGroup save(MarkerGroup markerGroup, MarkStage stage) {
        if (Objects.nonNull(markerGroup.getId())) {
            markerGroupLeaderRepo.deleteByWorkIdAndSubjectAndStageAndGroupId(markerGroup.getWorkId(), markerGroup.getSubject(), stage, markerGroup.getId());
        }
        Set<MarkUser> groupMarkers = markerGroup.getMarkers();
        List<MarkUser> markLeaders = groupMarkers.stream().filter(m -> Role.MARK_LEADER.equals(m.getRole())).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(markLeaders)) {
            throw new RuntimeException("每个分组至少有一个科组长");
        }
        MarkerGroup group = markerGroupRepo.save(markerGroup);
        List<MarkUser> markers = groupMarkers.stream().filter(m -> Role.MARKER.equals(m.getRole())).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(markers)) {
            List<MarkerGroupLeader> leaderList = new ArrayList<>();
            for (MarkUser markLeader : markLeaders) {
                for (MarkUser marker : markers) {
                    MarkerGroupLeader markerGroupLeader = new MarkerGroupLeader();
                    markerGroupLeader.setWorkId(markerGroup.getWorkId());
                    markerGroupLeader.setSubject(markerGroup.getSubject());
                    markerGroupLeader.setStage(stage);
                    markerGroupLeader.setGroupId(group.getId());
                    markerGroupLeader.setGroupName(group.getName());
                    markerGroupLeader.setMarkLeaderId(markLeader.getId());
                    markerGroupLeader.setMarkerId(marker.getId());
                    leaderList.add(markerGroupLeader);
                }
            }
            markerGroupLeaderRepo.save(leaderList);
        }

        return group;
    }

    /**
     * 删除分组
     *
     * @param markSubject
     * @param domain
     */
    @Transactional
    public void delete(MarkSubject markSubject, MarkerGroup domain) {
        int count = 0;
        if (MarkStage.LEVEL == markSubject.getStage() || MarkStage.SCORE == markSubject.getStage()) {
            count = markTaskRepo.countByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage());
        }
        if (count > 0) {
            throw new RuntimeException("当前阶段已发布任务，不能删除分组");
        }
        markerGroupRepo.delete(domain);

        markerGroupLeaderRepo.deleteByWorkIdAndSubjectAndStageAndGroupId(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage(), domain.getId());
    }

    /**
     * @param workId
     * @param subject
     * @param stage
     * @return
     */
    public List<Map<String, List<Long>>> listGroupLeaderAndMarkersByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage) {
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderRepo.findByWorkIdAndSubjectAndStage(workId, subject, stage);
        if (!CollectionUtils.isEmpty(markerGroupLeaders)) {
            Map<Long, List<MarkerGroupLeader>> maps = markerGroupLeaders.stream().collect(Collectors.groupingBy(MarkerGroupLeader::getGroupId));
            List<Map<String, List<Long>>> mapList = new ArrayList<>();
            for (Map.Entry<Long, List<MarkerGroupLeader>> listEntry : maps.entrySet()) {
                Map<String, List<Long>> map = new HashMap<>();
                // 科组长
                List<Long> markLeaderList = listEntry.getValue().stream().map(m -> m.getMarkLeaderId()).distinct().collect(Collectors.toList());
                map.put("markLeaders", markLeaderList);
                // 评卷员
                List<Long> markList = listEntry.getValue().stream().map(m -> m.getMarkerId()).distinct().collect(Collectors.toList());
                map.put("markers", markList);
                mapList.add(map);
            }
            return mapList;
        }
        return null;
    }

    /**
     * 根据评卷员，查询试卷
     *
     * @param workId
     * @param subject
     * @param stage
     * @param markers
     * @return
     */
    public List<Long> listPaperIdsByWorkIdAndSubjectAndStageAndMarkerIds(Long workId, Subject subject, MarkStage stage, List<Long> markers) {
        List<Paper> papers = markTaskRepo.findByWorkIdAndSubjectAndStageAndMarkerIdIn(workId, subject, stage, markers);
        if (!CollectionUtils.isEmpty(papers)) {
            List<Long> paperIds = papers.stream().map(m -> m.getId()).distinct().collect(Collectors.toList());
            return paperIds;
        }
        return null;
    }

    public List<Long> listPaperIdsByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage, List<Long> markers) {
        List<Object[]> papers = markTaskRepo.findByWorkIdAndSubjectAndStage(workId, subject.name(), stage.ordinal());
        if (!CollectionUtils.isEmpty(papers)) {
            List<Long> paperIds = papers.stream().filter(m -> markers.contains(Long.valueOf(m[1].toString()))).map(m -> Long.valueOf(m[0].toString())).distinct().collect(Collectors.toList());
            return paperIds;
        }
        return null;
    }

    /**
     * 根据科组长，查询评卷员
     *
     * @param workId
     * @param markSubject
     * @param markLeaderId
     * @return
     */
    public List<MarkerGroupLeader> listByWorkIdAndSubjectAndStageAndMarkLeaderId(Long workId, MarkSubject markSubject, Long markLeaderId) {
        return markerGroupLeaderRepo.findByWorkIdAndSubjectAndStageAndMarkLeaderId(workId, markSubject.getSubject(), markSubject.getStage(), markLeaderId);
    }

    /**
     * 查询分组
     *
     * @param workId
     * @param subject
     * @param stage
     * @return
     */
    public Set<Map<String, Object>> listGroupByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage) {
        stage = Objects.isNull(stage) ? MarkStage.LEVEL : stage;
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderRepo.findByWorkIdAndSubjectAndStage(workId, subject, stage);
        Set<Map<String, Object>> list = new HashSet<>();
        for (MarkerGroupLeader markerGroupLeader : markerGroupLeaders) {
            Map<String, Object> stringMap = new HashMap<>();
            stringMap.computeIfAbsent("groupId", v -> markerGroupLeader.getGroupId());
            stringMap.computeIfAbsent("groupName", v -> markerGroupLeader.getGroupName());
            list.add(stringMap);
        }
        return list;
    }

    /**
     * 根据分组查询组内评卷员
     *
     * @param workId
     * @param subject
     * @param level
     * @param groupId
     * @return
     */
    public List<Long> listByWorkIdAndSubjectAndStageAndGroupId(Long workId, Subject subject, MarkStage level, Long groupId) {
        if (Objects.isNull(groupId)) {
            return null;
        }
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderRepo.findByWorkIdAndSubjectAndStageAndGroupId(workId, subject, level, groupId);
        List<Long> markerIds = null;
        if (!CollectionUtils.isEmpty(markerGroupLeaders)) {
            markerIds = markerGroupLeaders.stream().map(m -> m.getMarkerId()).distinct().collect(Collectors.toList());
        }
        return markerIds;
    }
}
